#!/usr/bin/perl

use strict;

my $pdb_file = $ARGV[0];

# SARS-CoV-2 N-glycan positions
my @sarscov2_glycan_pos = (17, 61, 74, 122, 149, 165, 234, 282, 331, 343, 603, 616, 657, 709, 717, 801, 1074, 1098, 1134, 1158, 1173, 1194);
# SARS-CoV N-glycan positions
my @sarscov_glycan_pos = (29,65,73,109,118,119,158,227,269,318,330,357,589,602,691,699,783,1056,1080, 1116, 1140, 1155, 1176);

if ($pdb_file eq "") { die("usage: show_glycan_absent_present.pl pdb_file\n"); }

open(PDB, $pdb_file) || die("unable to open file: $pdb_file\n");
my @pdb_lines = <PDB>;
close(PDB);

# get all of the CA atoms from the file
my @ca_lines = ();
my %ca_found = ();
my %chain_found = ();
my @chains = ();
my $res_num = 0;
for (my $i = 0; $i < @pdb_lines; $i++)
{
    my $line = $pdb_lines[$i];
    my $res = substr($line, 21, 6);
    my $chn = substr($line, 21, 1);
    if ((substr($line, 0, 4) eq "ATOM") && (substr($line, 12, 4) eq " CA "))
    {
	if ($ca_found{$res} ne "") { next; }
	if ($chain_found{$chn} eq "") { push @chains, $chn; $chain_found{$chn} = 1; }
	$ca_lines[$res_num] = $line;
	$ca_found{$res} = 1;
	$res_num++;
    }
}

# look for glycan sequons
for (my $j = 0; $j < @chains; $j++)
{
    my $chain = $chains[$j];
    my $chain_output = "";
    my %glyc_found = ();
    my %new_glyc = ();
    my $min_res = 100000;
    my $max_res = 0;
    for (my $i = 0; $i < @ca_lines - 2; $i++)
    {
	if ((substr($ca_lines[$i], 21, 1) ne $chain) || (substr($ca_lines[$i], 21, 1) ne substr($ca_lines[$i+2], 21, 1))) { next; }

	my $line = $ca_lines[$i];
	my $line2 = $ca_lines[$i+1];
	my $line3 = $ca_lines[$i+2];

	my $res_num = int(substr($line, 22, 4));
	if ($res_num < $min_res) { $min_res = $res_num; }
	if ($res_num > $max_res) { $max_res = $res_num; }
	
	if (substr($line, 17, 3) ne "ASN") { next; }
	if (substr($line2, 17, 3) eq "PRO") { next; }
	if (((substr($line3, 17, 3) eq "SER") || (substr($line3, 17, 3) eq "THR")) &&
	    (substr($line, 21, 1) eq substr($line3, 21, 1)) &&
	    ((int(substr($line, 22, 4)) + 2) == (int(substr($line3, 22, 4)))))
	{ 
	    $glyc_found{$res_num} = 1;
	}
    }
    # now check which SARS-CoV-2 ones are present, in the range of the protein chain
    my $num_found = 0;
    for (my $k = 0; $k < @sarscov2_glycan_pos; $k++)
    {
	if (($sarscov2_glycan_pos[$k] < $min_res) || ($sarscov2_glycan_pos[$k] > $max_res)) { next; }
	my $n = $k + 1;
	if ($glyc_found{$sarscov2_glycan_pos[$k]} == 1)
	{
	    $chain_output .= "$chain\tN$n\t$sarscov2_glycan_pos[$k]\tPRESENT\n";
	    $glyc_found{$sarscov2_glycan_pos[$k]} = 2;
	    $num_found++;
	}
	else { $chain_output .= "$chain\tN$n\t$sarscov2_glycan_pos[$k]\tABSENT\n"; }
    }
    # checking for SARS-CoV glycan positions also
    if ($num_found == 0)
    {
	$chain_output = "";
	for (my $k = 0; $k < @sarscov_glycan_pos; $k++)
	{
	    if (($sarscov_glycan_pos[$k] < $min_res) || ($sarscov_glycan_pos[$k] > $max_res)) { next; }
	    my $n = $k + 1;
	    if ($glyc_found{$sarscov_glycan_pos[$k]} == 1)
	    {
		$chain_output .= "$chain\tN$n\t$sarscov_glycan_pos[$k]\tPRESENT\n";
		$glyc_found{$sarscov_glycan_pos[$k]} = 2;
		$num_found++;
	    }
	    else { $chain_output .= "$chain\tN$n\t$sarscov_glycan_pos[$k]\tABSENT\n"; }
	}
    }
    if ($num_found == 0) { next; } # this is probably not a spike chain, no glycans
    # check for additional glycans that are not ID'd as SARS-CoV-2
    foreach my $res (keys %glyc_found)
    {
	if ($glyc_found{$res} == 1) { $chain_output .= "NEW GLYCAN FOUND: $res\n"; }
    }
    print ${chain_output};
}
